REM *******************************************************************
REM *                                                                 *
REM *   SHEL4TO5 - A PROGRAM IN QBASIC 1.0 for DOS 5.0                *
REM *                                                                 *
REM *   By Carman Lawrick, Ottawa, Canada, June 18, 1991.             *
REM *                                                                 *
REM *   Reads DOS 4.0 menu files from the DOS 4.0 DOSSHELL            *
REM *   (.MEU files) and adds information found in them to            *
REM *   the DOS 5.0 DOSSHELL.INI file, thus porting                   *
REM *   the DOS 4.0 DOSSHELL menus to the new DOS 5.0 DOSSHELL.       *
REM *                                                                 *
REM *   To run the program, in DOS 5.0 type QBASIC /RUN SHEL4TO5      *
REM *                                                                 *
REM *******************************************************************

REM *********************************************************************
REM *                                                                   *
REM *  PROCEDURES                                                       *
REM *                                                                   *
REM *  MAIN                                                             *
REM *   DOFILE                                                     *
REM *       <DOFILE                                                *
REM *       OUTFIL                                                *
REM *            CAPIT                                            *
REM *   OUTFIL                                                     *
REM *                                                                   *
REM *                                                                   *
REM *  The main procedure calls outfile to initialize and to close      *
REM *  the input file DOSSHELL.INI and the output file TEMP.TMP.        *
REM *                                                                   *
REM *  It then calls DOFILE with the name of the root .MEU file in      *
REM *  the DOS 4.0 DOSSHELL tree, whose name is supplied to the MAIN    *
REM *  procedure by the user at program startup time.                   *
REM *                                                                   *
REM *  DOFILE opens the .MEU file as a random file several times        *
REM *  sometimes for                                                    *
REM *  two byte records, and sometimes for 1 byte records, depending    *
REM *  on whether pointers and tables of pointers are to be read, or    *
REM *  whether byte by byte data tables are to be read.                 *
REM *                                                                   *
REM *  By following the pointers in the .MEU files, data tables         *
REM *  containing menu entries are located, and data is extracted       *
REM *  from these into string variables.                                *
REM *                                                                   *
REM *  When a reference to a sub-menu is found in the .MEU file,        *
REM *  DOFILE calls itself (after closing the .MEU file) with a         *
REM *  parameter specifying a deeper indentation for output, and        *
REM *  specifying the file name of the sub-menu, instead of the         *
REM *  root .MEU file supplied by the user.  This recursive procedure   *
REM *  can keep calling itself again and again to handle very deep      *
REM *  levels of nested menus (provided memory available allows).       *
REM *                                                                   *
REM *  Once data for a menu entry is extracted, OUTFILE is called       *
REM *  with a command parameter, directing the OUTFILE routine to       *
REM *  add a program entry, or a group entry, or to close a group       *
REM *  entry.                                                           *
REM *                                                                   *
REM *  OUTFILE, at initialization, copies the DOSSHELL.INI file to      *
REM *  the TEMP.TMP file, up to the beginning of the main group.        *
REM *                                                                   *
REM *  At this point, an extra group heading is added if the old        *
REM *  DOS 4.0 main menu is to be made into a submenu here.             *
REM *                                                                   *
REM *  Having copied this much of the original DOSSHELL.INI file,       *
REM *  control is returned, so that DOFILE may add program and          *
REM *  group entries.                                                   *
REM *                                                                   *
REM *  When OUTFILE is directed to add a group entry, the command       *
REM *  line is scanned for dialog box parameters, and they are          *
REM *  replaced with consecutive variable ids: %1, %2, %3, etc.         *
REM *                                                                   *
REM *  When finished adding the program item, the command line          *
REM *  is again scanned for dialog box parameters, and this time        *
REM *  dialog commands are added to the output file as well.            *
REM *                                                                   *
REM *  DOFILE finishes processing the rood .MEU file, the main          *
REM *  procedure calls OUTFILE one more time for finishing up.          *
REM *                                                                   *
REM *  At this point, OUTFILE adds a group ending and/or beginning,     *
REM *  depending on whether the old and new main menus are to become    *
REM *  sub-menus or not.                                                *
REM *                                                                   *
REM *  The remainder of the menu(s) found in the DOSSHELL.INI file      *
REM *  are copied to the output file, and are followed by an additional *
REM *  group end, if the new menu is to become a sub-menu.              *
REM *                                                                   *
REM *  Copying from DOSSHELL.INI to the output file continues until     *
REM *  the color definitions block is found.  At this point, two new    *
REM *  color schemes are added, which are intended to approximate       *
REM *  the grey and cyan background color schemes that were provided    *
REM *  with the DOS 4.0 menu, but not in the DOS 5.0 menu.              *
REM *                                                                   *
REM *  Finally, the remainder of the DOSSHELL.INI file is copied to     *
REM *  the output TEMP.TMP file.  Now the program opens, consecutively, *
REM *  DOSSHELL.1, then DOSSHELL.2, then DOSSHELL.3, etc., each time    *
REM *  checking the size of the file opened.  When one is found with    *
REM *  a size of zero, that file is deleted, and the file name used     *
REM *  to rename the input DOSSHELL.INI file.  Now the TEMP.TMP file    *
REM *  is renamed to DOSSHELL.INI, and the program ends.                *
REM *                                                                   *
REM *                                                                   *
REM *********************************************************************

REM *******************************************************************
REM *                                                                 *
REM *   Declarations                                                  *
REM *                                                                 *
REM *******************************************************************

DECLARE SUB STRIPDLG (x$, Y$)
DECLARE SUB CAPIT (x$, Y$)
DECLARE SUB DOFILE (FILNM$, MARG%, GRPNAM$, HELPTXT$, PASS$)
DECLARE SUB OUTFIL (CMD%, TITL$, PASWORD$, COMANDS$, HELP$, MARG%)

REM *  GLOBAL VARIABLES  *
COMMON SHARED INPATH$, OUTPATH$, SUB4$, SUB5$, COLRAD$

DEFINT A-Z: CLS

REM *******************************************************************
REM *                                                                 *
REM *  Program greeting.                                              *
REM *                                                                 *
REM *******************************************************************


PRINT "SHEL4TO5 Convert DOS 4.0 SHELL files (.MEU) to DOS 5.0 SHELL Program List"
PRINT
PRINT "A utility program for DOS 5.0 by Carman Lawrick, Ottawa, Canada"
PRINT
PRINT

REM * Clear Keyboard Buffer
DO
	A$ = INKEY$
LOOP UNTIL A$ = ""

REM * Ask whether documentation is wanted now.
PRINT "Do you want to read documentaion on this program? (y/n/esc) ";
DO
	A$ = INPUT$(1)
	A = INSTR("YyNn" + CHR$(27), A$)
LOOP UNTIL A
PRINT A$
IF A$ = CHR$(27) THEN SYSTEM
IF INSTR("Yy", A$) THEN
	REM * Print in-line documentation.
	PRINT
	RESTORE HELPD
	READ A$
	DO
		PRINT A$
		READ A$
	LOOP UNTIL A$ = "ZZ"
	PRINT "Press any key for more...": A$ = INPUT$(1): CLS
	RESTORE HELPD2
	READ A$
	DO
		PRINT A$
		READ A$
	LOOP UNTIL A$ = "ZZ"
	SYSTEM
END IF
REM * Ask about how to combine the two main menus
PRINT
PRINT
PRINT "By bringing old DOS 4.0 SHELL menu entries to the new DOS 5.0 SHELL,"
PRINT "you will combine the old MAIN GROUP (or main menu) and the new MAIN GROUP."
PRINT
PRINT "You have four choices for how you want the old and new menus"
PRINT "to appear with each other."
PRINT
PRINT "    C  =  Combine both main menus into one larger one"
PRINT "    4  =  Use the old one for the main menu, and make"
PRINT "              the new menu into a sub-menu"
PRINT "    5  =  Use the new one for the main menu, and make"
PRINT "              the old menu into a sub-menu"
PRINT "    B  =  Make both main menus into sub-menus"
PRINT
PRINT "    L  =  Do not read old shell files at all,"
PRINT "              instead, add color choices ONLY."
PRINT
PRINT "Please type your choice  ( C, 4, 5, B, L, or esc) :"; : SUB4$ = "N": SUB5$ = "N": COLRAD$ = "N": COLRONLY$ = "N"
DO
	A$ = INPUT$(1)
	A = INSTR("cCbBlL45" + CHR$(27), A$)
LOOP UNTIL A
PRINT A$
IF A$ = CHR$(27) THEN SYSTEM
IF INSTR("Ll", A$) THEN
	COLRAD$ = "Y": COLRONLY$ = "Y"
ELSE
	IF INSTR("Bb", A$) THEN SUB4$ = "Y": SUB5$ = "Y"
	IF A$ = "4" THEN SUB5$ = "Y"
	IF A$ = "5" THEN SUB4$ = "Y"

	PRINT
	PRINT "Do you want DOS 4.0 color choices added to the DOS 5.0 DOSSHELL? (y/n): ";
	DO
		A$ = INPUT$(1)
		A = INSTR("YyNn" + CHR$(27), A$)
	LOOP UNTIL A
	PRINT A$
	IF A$ = CHR$(27) THEN SYSTEM
	COLRAD$ = "N": IF INSTR("Yy", A$) THEN COLRAD$ = "Y"

	PRINT
	PRINT
	PRINT "You may press CTRL-BREAK to abort any question:"
	PRINT
	PRINT


	LINE INPUT "Name of path containing old SHELL files [ press ENTER for C:\DOS\ ] "; A$
	A$ = LTRIM$(A$): A$ = RTRIM$(A$)
	IF A$ = "" THEN A$ = "C:\DOS\"
	IF MID$(A$, LEN(A$), 1) <> "\" THEN A$ = A$ + "\"
	INPATH$ = A$
END IF
LINE INPUT "Name of path containing DOS 5.0 [ press ENTER for C:\DOS\ ] "; A$
A$ = LTRIM$(A$): A$ = RTRIM$(A$)
IF A$ = "" THEN A$ = "C:\DOS\"
IF MID$(A$, LEN(A$), 1) <> "\" THEN A$ = A$ + "\"
OUTPATH$ = A$

REM * Call OUTFIL to get ready for new program and group items
CALL OUTFIL(1, "", "", "", "", 0)

IF COLRONLY$ <> "Y" THEN

	LINE INPUT "Name OLD Root Shell Menu File (without Extention) [default = SHELL] "; A$
	A$ = RTRIM$(A$): A$ = LTRIM$(A$)
	IF A$ = "" THEN A$ = "SHELL"
	IF INSTR(A$, ".") = 0 THEN A$ = A$ + ".MEU"
	PRINT : PRINT : PRINT "(You may press ESC at any time to abort this run)"
	PRINT "Reading and converting these DOS 4.0 Shell entries:": PRINT

	REM * Call DOFILE with the root .MEU file name in A$ to look for
	REM * program and group items, and add them to output file by calling OUTFILE
	REM *
	CALL DOFILE(A$, 0, "", "", "")
	REM * Since DOFILE calls itself again and again to cover nested menus,
	REM * control returns here when they are all finished, when the root .MEU
	REM * file is finished.

	REM * Now call OUTFILE to finish up, copying the rest of the original
	REM * DOSSHELL.INI file, inserting new color selections, and renaming
	REM * files.
END IF
CALL OUTFIL(5, "", "", "", "", 0)
SYSTEM



HELPD: REM
REM ************************************************************************
REM *                                                                      *
REM *  These data statements are the running documentation for the program *
REM *                                                                      *
REM ************************************************************************
DATA "Syntax: SHEL4TO5"
DATA " "
DATA "Use this program to read your DOS 4.0 .MEU files from a directory,"
DATA "and add their contents to the new DOS 5.0 SHELL. (DOSSHELL.INI)"
DATA "It also adds two new color scheme choices that approximate those"
DATA "available in the DOS 4.0 SHELL. (Called DOS 4.0 GREY and DOS 4.0 CYAN)"
DATA " "
DATA "You will find this useful if you have spent a lot of time setting up"
DATA "the Program Groups in the DOS 4.0 SHELL to provide a menu system"
DATA "for invoking your frequently used programs."
DATA " "
DATA "In switching to DOS 5.0 you will find (or have found) that"
DATA "the new SHELL system is not compatible with the old, and you"
DATA "have to rebuild all your program menu entries for the new"
DATA "SHELL by hand."
DATA " "
DATA "ZZ"
HELPD2: REM
DATA "This program asks for input and output directories,"
DATA "and the name of your root .MEU file, the one containing the"
DATA "Main Group in the DOS 4.0 Shell.  Defaults are offered."
DATA "The program then reads all referenced .MEU files, "
DATA "and adds them to your DOSSHELL.INI for use with DOS 5.0. "
DATA " "
DATA "If you have figured out how to make DOS 4.0 menus more than"
DATA "two menus deep, as I did, you will find that this program"
DATA "will support your nested menus and bring them intact to your"
DATA "DOS 5.0 menu."
DATA " "
DATA "Your old DOSSHELL.INI file is saved in DOSSHELL.x "
DATA "where x is a number, incremented each time you run"
DATA "this program.  Thus, your oldest backup will be DOSSHELL.1"
DATA "and your next to oldest will be DOSSHELL.2, etc."
DATA " "
DATA "Note that the DOS 4.0 Shell allowed over 400 characters in a single"
DATA "HELP entry, but the new DOS 5.0 Shell only allows 255 characters. "
DATA "HELP entries found in the .MEU files from DOS 4.0 will be truncated"
DATA "to the new limit of 255."
DATA "ZZ"
103
REM ************************************************************************
REM *                                                                      *
REM *  These data statements are the new color selections to be added      *
REM *  to the DOS 5.0 menu to approximate colors available in DOS 4.0 menu *
REM *                                                                      *
REM ************************************************************************
DATA "    selection = "
DATA "    {"
DATA "        title = DOS 4.0 Grey"
DATA "        foreground = "
DATA "        {"
DATA "            base = black"
DATA "            highlight = brightwhite"
DATA "            selection = brightcyan"
DATA "            alert = brightred"
DATA "            menubar = black"
DATA "            menu = black"
DATA "            disabled = white"
DATA "            accelerator = brown"
DATA "            dialog = black"
DATA "            button = black"
DATA "            elevator = brightwhite"
DATA "            titlebar = black"
DATA "            scrollbar = brightwhite"
DATA "            borders = black"
DATA "            drivebox = black"
DATA "            driveicon = black"
DATA "            cursor = brightwhite"
DATA "        }"
DATA "        background = "
DATA "        {"
DATA "            base = white"
DATA "            highlight = blue"
DATA "            selection = black"
DATA "            alert = brightwhite"
DATA "            menubar = brightyellow"
DATA "            menu = brightyellow"
DATA "            disabled = brightyellow"
DATA "            accelerator = brightyellow"
DATA "            dialog = cyan"
DATA "            button = brightyellow"
DATA "            elevator = brightwhite"
DATA "            titlebar = brightwhite"
DATA "            scrollbar = black"
DATA "            borders = black"
DATA "            drivebox = white"
DATA "            driveicon = white"
DATA "            cursor = white"
DATA "        }"
DATA "    }"
DATA "    selection = "
DATA "    {"
DATA "        title = DOS 4.0 Cyan"
DATA "        foreground = "
DATA "        {"
DATA "            base = black"
DATA "            highlight = brightwhite"
DATA "            selection = brightwhite"
DATA "            alert = brightred"
DATA "            menubar = black"
DATA "            menu = black"
DATA "            disabled = white"
DATA "            accelerator = brown"
DATA "            dialog = black"
DATA "            button = black"
DATA "            elevator = brightwhite"
DATA "            titlebar = black"
DATA "            scrollbar = brightwhite"
DATA "            borders = black"
DATA "            drivebox = black"
DATA "            driveicon = black"
DATA "            cursor = brightwhite"
DATA "        }"
DATA "        background = "
DATA "        {"
DATA "            base = cyan"
DATA "            highlight = blue"
DATA "            selection = black"
DATA "            alert = brightwhite"
DATA "            menubar = brightyellow"
DATA "            menu = brightyellow"
DATA "            disabled = brightyellow"
DATA "            accelerator = brightyellow"
DATA "            dialog = white"
DATA "            button = brightyellow"
DATA "            elevator = brightwhite"
DATA "            titlebar = brightwhite"
DATA "            scrollbar = black"
DATA "            borders = black"
DATA "            drivebox = cyan"
DATA "            driveicon = cyan"
DATA "            cursor = cyan"
DATA "        }"
DATA "    }
DATA "ZZ"

SUB CAPIT (x$, Y$)
REM ************************************************************************
REM *                                                                      *
REM *  This procedure coverts a string to capitals, and removes whitespace *
REM *  to prepare an input line for case insensitive, free-format          *
REM *  keyword detection.                                                  *
REM *                                                                      *
REM ************************************************************************
Y$ = "": IF x$ = "" THEN EXIT SUB
FOR I = 1 TO LEN(x$)
A$ = MID$(x$, I, 1)
IF INSTR(" " + CHR$(9) + CHR$(15), A$) = 0 THEN
	IF (A$ >= "a") AND (A$ <= "z") THEN A$ = CHR$(ASC(A$) - 32)
	Y$ = Y$ + A$
END IF
NEXT
EXIT SUB
END SUB

SUB DOFILE (FILNM$, MARG, GNAM$, HLPS$, GPAS$)
REM ************************************************************************
REM *                                                                      *
REM *  This procedure reads a DOS 4.0 DOSSHELL .MEU file and extracts      *
REM *  from it, program and group items, together with the help text       *
REM *  associated with each.                                               *
REM *                                                                      *
REM *  The procedure is passed the name of the .MEU file to read, and      *
REM *  when encountering a group item, calls itself with the name of the   *
REM *  file containing the program group.                                  *
REM *                                                                      *
REM *  A DOS 4.0 DOSSHELL .MEU file is composed of data blocks, which are  *
REM *  organized through pointer tables within the .MEU file.              *
REM *  A description of the stucture of a .MEU file follows:               *
REM *                                                                      *
REM *  The file begins with a two byte word, containing the binary digits  *
REM *  1234.  Since an Intel word is read and written with the low byte    *
REM *  first, and then the high byte following it, the file begins with    *
REM *  3412h.  This seems to be a validation signature, to be checked,     *
REM *  before processing, to make sure that a valid DOSSHELL .MEU file     *
REM *  is being read.                                                      *
REM *                                                                      *
REM *  Next comes a table of 8 byte entries which I call a Table of        *
REM *  Contents.  As near as I can tell, only the first two entries        *
REM *  in this table are used, and their structure is as follows:          *
REM *                                                                      *
REM *                          EG.                                         *
REM *      OFFSET  SIZE/TYPE  VALUE  MEANING                               *
REM *                                                                      *
REM *         0      WORD            - entry type:                         *
REM *                         00FEh  = this entry points to a help index   *
REM *                         00F7h  = this entry points to a menu item index *
REM *                                                                      *
REM *         2      WORD     0      - unknown (always 0)                  *
REM *                                                                      *
REM *         4      WORD     00A4h  - pointer to the index, offset from BOF *
REM *                                                                      *
REM *         6      WORD     0010h  - number of 8 byte entries in the index *
REM *                                                                      *
REM *                                                                      *
REM *  A Help Index, pointed at by a Table of Contents entry type FE       *
REM *  consists of 8 byte entries, each pointing to a block of help text.  *
REM *  The Help Index begins with a two word header.  The first word shows *
REM *  the number of valid help text entries contained in the index.       *
REM *  The second word seems to be unused, and contains a binary one.      *
REM *  The structure of each help text entry that follows is thus:         *
REM *                                                                      *
REM *                          EG.                                         *
REM *      OFFSET  SIZE/TYPE  VALUE  MEANING                               *
REM *                                                                      *
REM *         0      WORD     0001h  - Help entry number, each is unique   *
REM *         2      WORD     0000   - Seems to be unused                  *
REM *         4      WORD     0138h  - pointer to the help text block,     *
REM *                                  offset from Beginning Of File (BOF) *
REM *         6      WORD     01E0h  - Length of help text block           *
REM *                                                                      *
REM *                                                                      *
REM *  A Help Text Block begins with a signature of 0CDh, and is padded    *
REM *  with blanks to the length of the block.  The unique help entry      *
REM *  number is referenced by its corresponding program or group menu     *
REM *  item, as mentioned below.  This number ties a help text entry       *
REM *  to its corresponding program or group menu item.                    *
REM *                                                                      *
REM *  A Menu Item Index, pointed at by a Table of Contents entry type F7  *
REM *  also consists of 8 byte entries, each pointing to a Menu Item       *
REM *  Block.  The Menu Item Index also begins with a two word header.     *
REM *  The first word shows the number of entries that have been used      *
REM *  in the Menu Item Index.  The second word shows how many of these    *
REM *  entries is valid (undeleted).  Here is the structure of each        *
REM *  Menu Item Index entry:                                              *
REM *                                                                      *
REM *                          EG.                                         *
REM *      OFFSET  SIZE/TYPE  VALUE  MEANING                               *
REM *                                                                      *
REM *        0       WORD     0003h  - Display sequence number.  Shows     *
REM *                                  the order in which menu items       *
REM *                                  are displayed on the menu.          *
REM *                                  A Group Reorder operation changes   *
REM *                                  these values.  Must be zero if the  *
REM *                                  menu item is deleted.               *
REM *                                                                      *
REM *        2       WORD     0000h  - Unknown.  Seems unused.             *
REM *        4       WORD     039Ch  - Pointer to the Menu Item Block,     *
REM *                                  offset from Beginning Of File (BOF) *
REM *        6       WORD     022Ch  - Length of the Menu Item Block.      *
REM *                                                                      *
REM *                                                                      *
REM *  A Menu Item Block consists of both binary flags and ASCII character *
REM *  strings.  It may refer to any of three types of menu items.         *
REM *  It may refer to a DOSSHELL Internal Function such as                *
REM *  the File Manager, Change Colors, or the DOS Prompt.  It may also    *
REM *  refer to a sub-menu or program group, or it may refer to a          *
REM *  program item, which will run a specific command list.               *
REM *  The structure of a Menu Item Block is as follows:                   *
REM *                                                                      *
REM *                          EG.                                         *
REM *      OFFSET  SIZE/TYPE  VALUE  MEANING                               *
REM *                                                                      *
REM *         0      BYTE   0 OR 20h - unknown                             *
REM *                                                                      *
REM *         1    40 BYTES          - Menu Item Title - to be shown       *
REM *                                  on the menu                         *
REM *         41     BYTE            - Menu Item Type Flag:                *
REM *                          00    = Subgroup, Submenu Item              *
REM *                          01    = Program Item                        *
REM *                                                                      *
REM *         42   8 BYTES           - Password or 8 blanks, not encrypted *
REM *                                                                      *
REM *         50     WORD     0001h  - Associated help text entry number   *
REM *                                                                      *
REM *         52                     - Different contents for different    *
REM *                                  Menu Item Types:                    *
REM *                                                                      *
REM *                BYTES           = when Menu Item Type Flag, byte 41   *
REM *                                  is 00, indicating a subgroup,       *
REM *                                  here starts the blank terminated    *
REM *                                  file name of the .MEU file          *
REM *                                  containing the subgroup.            *
REM *                                                                      *
REM *                DWORD 000A0032h = when Menu Item Type Flag, byte 41   *
REM *                                  is 01, indicating a program item,   *
REM *                                  here is a DOUBLE WORD of unknown    *
REM *                                  meaning.                            *
REM * when byte 41 is 01:                                                  *
REM *                                                                      *
REM *         56     BYTES           = when Menu Item Type Flag, byte 41   *
REM *                                  is 01, indicating a program item,   *
REM *                                  here is one or more bytes indicating*
REM *                                  the command(s) to run:              *
REM *                                                                      *
REM *                           0FCh = DOSSHELL Internal Function for      *
REM *                                  Command Prompt.                     *
REM *                                                                      *
REM *                           0FDh = DOSSHELL Internal Function for      *
REM *                                  File System.                        *
REM *                                                                      *
REM *                           0FEh = DOSSHELL Internal Function for      *
REM *                                  Change Colors.                      *
REM *                                                                      *
REM *                         STRING = Command or Command list to run.     *
REM *                                  Commands are separated with 0BAh.   *
REM *                                                                      *
REM *                                                                      *
REM *                                                                      *
REM ************************************************************************

FLN = 3: REM Use file #3 for the .MEU files.
IF GNAM$ <> "" THEN PRINT SPACE$(MARG - 4); "Group: "; GNAM$ 'tell user whats happening (unless main group)
IF GNAM$ <> "" THEN CALL OUTFIL(2, GNAM$, GPAS$, "", HLPS$, MARG) 'start this group (unless main group)
GOSUB OPN2  'open the .MEU file for reading a pointer at a time - pointers are one word (2 bytes) long

REM TEST SIGNATURE:
IF LOF(FLN) = 0 THEN
	CLOSE FLN: KILL INPATH$ + FILNM$
ELSE
	GOSUB ESCCHK: GET #FLN, 1: A = CVI(IN$)
	IF A = &H1234 THEN

		REM FIND NUMBER OF TOCS
		GOSUB ESCCHK: GET #FLN, 2: TOCS = CVI(IN$)

		REM GO THROUGH TOCS TO FIND HELP INDEX
		HLPNXOF = 0
		FOR I = 1 TO TOCS: TOCAD = ((I - 1) * 4) + 3
			GOSUB ESCCHK: GET #FLN, TOCAD: ID = CVI(IN$): GOSUB ESCCHK: GET #FLN, TOCAD + 2: OFS = CVI(IN$): GOSUB ESCCHK: GET #FLN, TOCAD + 3: ENTRS = CVI(IN$)
			IF ID = &HFE THEN NOHLPNX = ENTRS: HLPNXOF = OFS + 4: EXIT FOR
		NEXT

		REM GO THROUGH TOCS TO FIND TITLE INDEX

		FOR I = 1 TO TOCS: TOCAD = ((I - 1) * 4) + 3
			GOSUB ESCCHK: GET #FLN, TOCAD: ID = CVI(IN$): GOSUB ESCCHK: GET #FLN, TOCAD + 2: OFS = CVI(IN$): GOSUB ESCCHK: GET #FLN, TOCAD + 3: ENTRS = CVI(IN$)
			IF ID = &HF7 THEN GOSUB GOTITLE
		NEXT
	END IF
	CLOSE FLN
END IF
IF GNAM$ <> "" THEN PRINT SPACE$(MARG - 4); "EndGroup: "; GNAM$
IF GNAM$ <> "" THEN CALL OUTFIL(3, GNAM$, GPAS$, "", HLPS$, MARG)
EXIT SUB

REM GET NUMBER ENTRIES IN TITLE INDEX
GOTITLE: REM
TOFS = (OFS / 2) + 1
GOSUB ESCCHK: GET #FLN, TOFS: TIXX = CVI(IN$): GOSUB ESCCHK: GET #FLN, TOFS + 1: TIXA = CVI(IN$)

REM GO THROUGH TITLE INDEX
FOR IXSEQ = 1 TO TIXA
	FOR IXJ = 1 TO ENTRS
		TIXAD = ((IXJ - 1) * 4) + TOFS + 2
		GOSUB ESCCHK: GET #FLN, TIXAD: TISEQ = CVI(IN$): GOSUB ESCCHK: GET #FLN, TIXAD + 2: BLOCKOFS = CVI(IN$): GOSUB ESCCHK: GET #FLN, TIXAD + 3: BLOCKLEN = CVI(IN$)
		IF TISEQ = IXSEQ THEN GOSUB GOTBLOCK: EXIT FOR
	NEXT
NEXT
RETURN

GOTBLOCK: REM PROCESS TITLE BLOCK
CLOSE FLN: GOSUB OPN1
BLKOFS = BLOCKOFS + 1
BLKLEN = BLOCKLEN

REM GET TITLE STRING
TITN$ = "": FOR I = 1 TO 40: GOSUB ESCCHK: GET #FLN, BLKOFS + I: TITN$ = TITN$ + IN$: NEXT
TITN$ = LTRIM$(TITN$): TITN$ = RTRIM$(TITN$)
IF TITN$ = "" THEN TITN$ = " "

REM GET ENTRY TYPE: GROUP OR PROGRAM
GOSUB ESCCHK: GET #FLN, BLKOFS + 41: TITYP = ASC(IN$): IF TITYP = 0 THEN CMDPOS = 52 ELSE CMDPOS = 56

REM GET PASSWORD
PASS$ = "": FOR I = 42 TO 49: GOSUB ESCCHK: GET #FLN, BLKOFS + I: PASS$ = PASS$ + IN$: NEXT
PASS$ = LTRIM$(PASS$): PASS$ = RTRIM$(PASS$)

REM GET COMMAND STRING - IGNORE DOS 4.0 DOSSHELL INTERNAL FUNCTIONS.
GOSUB ESCCHK: GET #FLN, BLKOFS + CMDPOS
IF IN$ = CHR$(&HFC) THEN CMDST$ = "[COMMAND PROMPT]": CLOSE FLN: GOSUB OPN2: RETURN
IF IN$ = CHR$(&HFD) THEN CMDST$ = "[FILE SYSTEM]": CLOSE FLN: GOSUB OPN2: RETURN
IF IN$ = CHR$(&HFE) THEN CMDST$ = "[CHANGE COLORS]": CLOSE FLN: GOSUB OPN2: RETURN
CMDST$ = ""
FOR I = CMDPOS TO (BLKLEN - 1)
	GOSUB ESCCHK: GET #FLN, BLKOFS + I
	IF IN$ = CHR$(0) THEN EXIT FOR
	IF IN$ = CHR$(&HBA) THEN CMDST$ = CMDST$ + " ;": LSET IN$ = " "
	CMDST$ = CMDST$ + IN$
NEXT
CMDST$ = RTRIM$(CMDST$): CMDST$ = LTRIM$(CMDST$)
IF RIGHT$(CMDST$, 2) = " ;" THEN CMDST$ = LEFT$(CMDST$, LEN(CMDST$) - 2)

REM GET HELP
HLPSTR$ = ""
IF HLPNXOF <> 0 THEN
	CLOSE FLN: GOSUB OPN2
	GOSUB ESCCHK: GET #FLN, (BLKOFS + 50) / 2: HELPNUM = CVI(IN$)
	IF HELPNUM <> 0 THEN
		FOR I = 1 TO NOHLPNX: RECN = ((HLPNXOF + ((I - 1) * 8)) / 2) + 1
		GOSUB ESCCHK: GET #FLN, RECN: HINXN = CVI(IN$): GOSUB ESCCHK: GET #FLN, RECN + 2: HOFS = CVI(IN$): GOSUB ESCCHK: GET #FLN, RECN + 3: HLEN = CVI(IN$)
		IF HINXN = HELPNUM THEN EXIT FOR
		NEXT
		IF HINXN = HELPNUM THEN
			CLOSE FLN: GOSUB OPN1
			FOR I = 1 TO HLEN - 2
				GOSUB ESCCHK: GET #FLN, I + HOFS + 1: IF ASC(IN$) = 0 THEN EXIT FOR
				IF ASC(IN$) > 127 THEN
					HLPSTR$ = "": EXIT FOR
				ELSE
					IF IN$ = "&" THEN HLPSTR$ = HLPSTR$ + "^": LSET IN$ = "m"
					HLPSTR$ = HLPSTR$ + IN$
				END IF
			NEXT
			HLPSTR$ = LTRIM$(HLPSTR$): HLPSTR$ = RTRIM$(HLPSTR$)
			IF LEN(HLPSTR$) > 255 THEN HLPSTR$ = LEFT$(HLPSTR$, 255)
			END IF
	END IF
END IF

IF TITYP = 0 THEN
	CLOSE FLN
	CALL DOFILE(CMDST$, MARG + 4, TITN$, HLPSTR$, PASS$)
	GOSUB OPN1
ELSE
	PRINT SPACE$(MARG); "Program: "; TITN$
	CALL OUTFIL(4, TITN$, PASS$, CMDST$, HLPSTR$, MARG)
END IF

INTERF: REM
CLOSE FLN: GOSUB OPN2
RETURN


ESCCHK: A$ = INKEY$: IF A$ = CHR$(27) THEN SYSTEM ELSE RETURN

OPN2: OPEN INPATH$ + FILNM$ FOR RANDOM AS FLN LEN = 2
FIELD #FLN, 2 AS IN$: RETURN
OPN1: OPEN INPATH$ + FILNM$ FOR RANDOM AS FLN LEN = 1
FIELD #FLN, 1 AS IN$: RETURN

EROUT: PRINT : PRINT "PROGRAM ABORTED": SYSTEM
END SUB

SUB OUTFIL (CMD, TITL$, PASWORD$, COMANDS$, HELP$, MARG) STATIC

SELECT CASE CMD

CASE 1
	REM INITIALIZATION OF OUTPUT FILE
	OPEN OUTPATH$ + "DOSSHELL.INI" FOR INPUT AS 1
	OPEN OUTPATH$ + "TEMP.TMP" FOR OUTPUT AS 2
	DO
		IF EOF(1) THEN PRINT "Couldn't find   group =   in DOSSHELL.INI": SYSTEM
		GOSUB ESCCHK1: LINE INPUT #1, A$
		CALL CAPIT(A$, B$)
		A = INSTR(B$, "GROUP=")
		IF A = 1 THEN FINDGRP = -1
		IF FINDGRP THEN
			A$ = RTRIM$(A$): A$ = LTRIM$(A$)
			A = INSTR(A$, "{")
			IF A THEN
				IF A < LEN(A$) THEN ORGGRP$ = MID$(A$, A + 1) ELSE ORGGRP$ = ""
				A$ = LEFT$(A$, A)
				PRINT #2, A$
				IF SUB4$ = "Y" THEN PRINT #2, "group = "
				IF SUB4$ = "Y" THEN PRINT #2, "{"
				IF SUB4$ = "Y" THEN PRINT #2, "title = DOS 4.0 Menu"
				EXIT SUB
			END IF
		END IF
		PRINT #2, A$
	LOOP

CASE 2
	REM START A NEW GROUP
	PRINT #2, SPACE$(MARG); "group =": PRINT #2, SPACE$(MARG); "{"
	PRINT #2, SPACE$(MARG + 4); "title = "; TITL$
	IF HELP$ <> "" THEN PRINT #2, SPACE$(MARG + 4); "help = "; HELP$
	IF PASWORD$ <> "" THEN PRINT #2, SPACE$(MARG + 4); "password = "; PASWORD$
	EXIT SUB

CASE 3
	REM PROCESS END OR GROUP
	PRINT #2, SPACE$(MARG); "}": EXIT SUB


CASE 4
	REM PROCESS NEW PROGRAM ENTRY
	REM STRIP COMMAND STRING OF DIALOG STUFF, REPLACING WITH %1, %2, ETC.
	Z = 0: Y$ = COMANDS$
       
	DO
		Z = Z + 1: A = INSTR(Y$, "["): IF A = 0 THEN EXIT DO
		B = INSTR(Y$, "]"): IF B = 0 THEN EXIT DO
		Z$ = "": IF A > 1 THEN Z$ = LEFT$(Y$, A - 1)
		Z$ = Z$ + "%" + MID$(STR$(Z), 2)
		IF B < LEN(Y$) THEN Y$ = Z$ + MID$(Y$, B + 1) ELSE Y$ = Z$
	LOOP
	PRINT #2, SPACE$(MARG + 4); "program =": PRINT #2, SPACE$(MARG + 4); "{"
	PRINT #2, SPACE$(MARG + 8); "title = "; TITL$
	IF Y$ <> "" THEN PRINT #2, SPACE$(MARG + 8); "command = "; Y$
	IF PASWORD$ <> "" THEN PRINT #2, SPACE$(MARG + 8); "password = "; PASWORD$
	IF HELP$ <> "" THEN PRINT #2, SPACE$(MARG + 8); "help = "; HELP$
	PRINT #2, SPACE$(MARG + 8); "pause = disabled"


OUTDLG: REM GO THROUGH COMMAND STRING AGAIN, CONVERTING DIALOG PROMPT STUFF
	Z = 0: Y$ = COMANDS$

	DO
		IF Y$ = "" THEN EXIT DO
		A = INSTR(Y$, "["): IF A = 0 THEN EXIT DO
		B = INSTR(Y$, "]"): IF B = 0 THEN EXIT DO
		Z = Z + 1: Z$ = MID$(Y$, A + 1, B - A - 1)
		IF B < LEN(Y$) THEN Y$ = MID$(Y$, B + 1) ELSE Y$ = ""
		PRINT #2, SPACE$(MARG + 8); "dialog =": PRINT #2, SPACE$(MARG + 8); "{"
		DO
			IF Z$ = "" THEN EXIT DO
			A = INSTR(Z$, "/"): IF A = 0 THEN EXIT DO
			DCMD = INSTR("TIPD", MID$(Z$, A + 1, 1))
			IF DCMD = 0 THEN DCMD = INSTR("tipd", MID$(Z$, A + 1, 1))
			A = INSTR(Z$, CHR$(34)): IF A = 0 THEN EXIT DO
			IF A < LEN(Z$) THEN B = INSTR(A + 1, Z$, CHR$(34)) ELSE EXIT DO
			IF B = 0 THEN B = LEN(Z$)
			PARM$ = MID$(Z$, A + 1, B - A - 1)
			IF B < LEN(Z$) THEN Z$ = MID$(Z$, B + 1) ELSE Z$ = ""
			IF DCMD <> 0 THEN PRINT #2, SPACE$(MARG + 12); RTRIM$(MID$("      title  info   prompt default", DCMD * 7, 7)); " = "; PARM$
		LOOP
	PRINT #2, SPACE$(MARG + 12); "parameter = %" + MID$(STR$(Z), 2): PRINT #2, SPACE$(MARG + 8); "}"
	LOOP
	PRINT #2, SPACE$(MARG + 4); "}"
	EXIT SUB

CASE 5
	REM END OF INPUT FILES, FINISH OUTPUT FILE
	REM Copy the rest of the DOSSHELL.INI menu items, and insert color selections
	BRCEND = 0
	IF SUB4$ = "Y" THEN PRINT #2, "}"
	IF SUB5$ = "Y" THEN PRINT #2, "group = "
	IF SUB5$ = "Y" THEN PRINT #2, "{"; ORGGRP$: CNT = 1: CNTON = -1
	IF SUB5$ = "Y" THEN PRINT #2, "title = Original DOS 5.0 Menu"
	IF COLRAD$ = "N" THEN COLRD = -1

	DO
		TMPF = 0: IF EOF(1) THEN TMPF = -1: IF COLRD = 0 THEN PRINT "Couldn't find COLOR = ": SYSTEM
		IF TMPF THEN IF (BRCEND = 0) AND (SUB5$ = "Y") THEN PRINT "Couldn't find end brace for main group": SYSTEM
		IF TMPF THEN EXIT DO
		GOSUB ESCCHK1: LINE INPUT #1, A$: A = 0
       
		DO
			A = INSTR(A + 1, A$, "{"): IF A THEN CNT = CNT + 1
		LOOP UNTIL A = 0
	       
		A = 0
		DO
			A = INSTR(A + 1, A$, "}"): IF A THEN CNT = CNT - 1
		LOOP UNTIL A = 0
       
		CALL CAPIT(A$, B$)
		A = INSTR(B$, "COLOR=")
		IF (A = 1) AND (COLRAD$ = "Y") THEN
			REM WHEN COLOR BLOCK ENCOUNTERED THIS INSERTS NEW COLOR SELECTIONS
			COLRD = -1
			DO
				REM A$ = RTRIM$(A$): A$ = LTRIM$(A$)

				REM COPY COLOR BLOCK LINES TILL ENCOUNTER END BRACE AT END OF COLOR BLOCK
				IF A$ <> "" THEN
					FOR I = 1 TO LEN(A$): IF MID$(A$, I, 1) = "{" THEN CLBKS = CLBKS + 1: CLBKON = -1
					IF MID$(A$, I, 1) = "}" THEN CLBKS = CLBKS - 1
						REM ONCE END BRACE AT END OF COLOR BLOCK ENCOUNTERED, ADD COLOR CHOICES
						IF (CLBKON) AND (CLBKS = 0) THEN
							ORGCLR$ = MID$(A$, I): IF I > 1 THEN A$ = LEFT$(A$, I - 1): PRINT #2, A$
							PRINT
							PRINT "Adding color choice: DOS 4.0 Grey"
							PRINT "Adding color choice: DOS 4.0 Cyan"
							PRINT
							RESTORE 103
							DO
								READ A$: IF A$ = "ZZ" THEN PRINT #2, ORGCLR$: EXIT DO
								PRINT #2, A$
							LOOP
							EXIT DO
						END IF
					NEXT
				END IF
				PRINT #2, A$
				IF EOF(1) THEN PRINT "Unmatched braces around COLOR block found in input DOSSHELL.INI": SYSTEM
				LINE INPUT #1, A$
			LOOP
		ELSE
			PRINT #2, A$
		END IF
		IF CNTON AND CNT = 0 THEN CNTON = 0: PRINT #2, "}": BRCEND = -1
	LOOP


	REM CLOSE FILES, AND RENAME THEM
	CLOSE 1, 2
	A = 0
	FOR I = 1 TO 999: OLDFNAM$ = OUTPATH$ + "DOSSHELL." + MID$(STR$(I), 2)
		OPEN OLDFNAM$ FOR RANDOM AS 1 LEN = 1
		A# = LOF(1): CLOSE 1
		IF A# = 0 THEN A = -1: EXIT FOR
	NEXT
       
	IF A = 0 THEN PRINT "Could not rename old DOSSHELL.INI for backup": SYSTEM
       
	KILL OLDFNAM$: NAME OUTPATH$ + "TEMP.TMP" AS OUTPATH$ + "DOSSHELL.TMP"
	NAME OUTPATH$ + "DOSSHELL.INI" AS OLDFNAM$
	NAME OUTPATH$ + "DOSSHELL.TMP" AS OUTPATH$ + "DOSSHELL.INI"
	PRINT : PRINT : PRINT "New DOSSHELL.INI created, old one backed up in "; OLDFNAM$
	EXIT SUB

CASE ELSE
	REM CALLED WITH INVALID COMMAND
	EXIT SUB
END SELECT


ESCCHK1: A$ = INKEY$: IF A$ = CHR$(27) THEN SYSTEM ELSE RETURN

END SUB

